/******************************************************************
 *****                                                        *****
 *****  Name: EasyWeb.c                                       *****
 *****  Ver.: 1.0                                             *****
 *****  Date: 07/05/2001                                      *****
 *****  Auth: Andreas Dannenberg                              *****
 *****        HTWK Leipzig                                    *****
 *****        university of applied sciences                  *****
 *****        Germany                                         *****
 *****  Func: implements a dynamic HTTP-server by using       *****
 *****        the easyWEB-API                                 *****
 *****                                                        *****
 ******************************************************************/

//*****************************************************************
//	modifications by Cam Thompson, Micromega Corporation
//	January 18, 2012
//*****************************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "LPC43xx.h"
#include "lpc43xx_scu.h"
#include "web_system_LPC43xx.h"
#include "ipc_memory.h"
#include "debug_frmwrk.h"

#define extern            // Keil: Line added for modular project management

#include "EasyWeb.h"
#include "emac.h"
#include "tcpip.h"
#include "webpage.h"

#define IRC_OSC     (12000000UL)        /* Internal RC oscillator frequency   */
uint32_t SystemFrequency = IRC_OSC * 5; /*!< System Clock Frequency (Core Clock)  */
uint32_t AHBSystemFrequency = IRC_OSC * 5;

#define MAX_PAGE_LEN 4000
char webSide[MAX_PAGE_LEN];

extern void TCPClockHandler(void);

volatile unsigned long TimeTick  = 0;

#define FRAMESIZE	60	//1514

void M0_RIT_OR_WWDT_IRQHandler(void);

#define MAX_SAMPLES	10
struct sampleRecord
{
	int	num;
	int	date;
	int	time;
	int	temp;
	int	adc;
} sampleData[MAX_SAMPLES];

int sampleIndex = 0;
int lastSampleCount = 0;

//-------------------------- Local Prototypes ----------------------------------

void ReadSample(void);
void HTTPServer(void);
void ReplaceTokens(char *strOut, const char *strIn);
char *CreateHistory(char *sptr);
char *CopyDateTime(char *sptr, int date, int time);
char *CopyString(char *sptr, char *from);

//==============================================================================
// web clock tick
//==============================================================================

void web_tick(void)
{
	TimeTick++;
	if (TimeTick >= 20)
	{
		M0_COUNTER = M0_COUNTER + 1;			// increment M0 interrupt counter

		#if (PLATFORM == NXP_VALIDATION_BOARD)	// toggle M0 LED
			LPC_GPIO_PORT->NOT[5] |= (1UL << 19);
		#endif
		#if (PLATFORM == HITEX_A2_BOARD)
			LPC_GPIO_PORT->NOT[4] |= (1UL << 1);
		#endif

		TimeTick = 0;
		TCPClockHandler();
	}
}

//==============================================================================
// Initialize board specific I/O
//==============================================================================

void IOInit(void)
{
	scu_pinmux(0x9 ,2 , MD_PDN, FUNC2); 	// GPIO4_14: LD11
	scu_pinmux(0xA ,4 , MD_PDN, FUNC2); 	// GPIO4_11: LD10
	scu_pinmux(0x6 ,10 , MD_PUP, FUNC3);	// GPIO3_6: button 0
	scu_pinmux(0x4 ,0 , MD_PUP, FUNC0); 	// GPIO2_0: button 1

	LPC_GPIO4->DIR |= (1<<14);			// LD11 = output
	LPC_GPIO4->DIR |= (1<<11);			// LD11 = output
	LPC_GPIO3->DIR &= ~(1<<6);			// Button 0 = input
	LPC_GPIO2->DIR &= ~(1<<0);			// Button 1 = input
}

//==============================================================================
// main entry and setup for EasyWeb server
//==============================================================================

int web_main(void)
{
	int i;
#ifdef CORE_M0
#else
	SystemInit();
	SetPL160M(SRC_IRC, 6); 						// Set PL160M @ 72 MHz
	SetClock(BASE_M4_CLK, SRC_PL160M_0, DIV1);	// Run base M3 clock from PL160M, div by 1 = no division
	IOInit();
#endif
	LPC_CREG->CREG0 |= (1<<1)|(1<<0);			// Enable 32 kHz & 1 kHz on osc32k

#if MII
 	SetClock(BASE_PHY_TX_CLK, SRC_ENET_TX_CLK, DIV1);
 	SetClock(BASE_PHY_RX_CLK, SRC_ENET_RX_CLK, DIV1);
#else
	SetClock(BASE_PHY_TX_CLK, SRC_ENET_TX_CLK, DIV2);
	SetClock(BASE_PHY_RX_CLK, SRC_ENET_TX_CLK, DIV2);
#endif

#ifdef CORE_M0
	LPC_RITIMER->COMPVAL = SystemFrequency/100; 	// Set match value
	LPC_RITIMER->COUNTER=0;							// Set count value to 0
	LPC_RITIMER->CTRL = (1<<3)|(1<<2)|(1<<1)|(1<<0);// Enable, enable break, reset on mask compare, clear interrupt flag

	NVIC_EnableIRQ(M0_RITIMER_OR_WWDT_IRQn);
#else
	SysTick_Config(SystemFrequency/100);			// Generate interrupt every 10 ms
#endif

	// initialize the sample data history
	for (i = 0; i < MAX_SAMPLES; i++)
		sampleData[i].num = 0;

	TCPLowLevelInit();

	HTTPStatus = 0;									// clear HTTP-server's flag register

	TCPLocalPort = TCP_PORT_HTTP;					// set port we want to listen to

	while (1)										// repeat forever
	{
		if (!(SocketStatus & SOCK_ACTIVE))
		{
			TCPPassiveOpen();						// listen for incoming TCP-connection
		}

    	DoNetworkStuff();							// handle network and easyWEB-stack event

 		ReadSample();

    	HTTPServer();

#ifdef CORE_M0
		if (LPC_RITIMER->CTRL & 1) {
			if (LPC_RITIMER->CTRL & 1) {
				if (LPC_RITIMER->CTRL & 1) {
					if (LPC_RITIMER->CTRL & 1) {
						M0_RIT_OR_WWDT_IRQHandler();
					}
				}
			}
		}
#endif

	}
}

//==============================================================================
// read next sample
//==============================================================================

void ReadSample(void)
{
	int sampleCount, temp, tempTotal, adc, adcTotal, i, n;

   	sampleCount = ipc_GetValue(IPC_SAMPLE_COUNT);
	if (sampleCount != lastSampleCount)
	{
		sampleIndex = (sampleIndex + 1) % MAX_SAMPLES;

		ipc_StartRead();
		sampleData[sampleIndex].num = sampleCount;
		sampleData[sampleIndex].date = ipc_GetValue(IPC_SAMPLE_DATE);
		sampleData[sampleIndex].time = ipc_GetValue(IPC_SAMPLE_TIME);
		temp = ipc_GetValue(IPC_TEMP); 
		sampleData[sampleIndex].temp = temp;
		adc = ipc_GetValue(IPC_ADC);
		sampleData[sampleIndex].adc = adc;
		ipc_EndRead();

 		// get temperature min/max
		if (lastSampleCount == 0 || temp < ipc_GetValue(IPC_MIN_TEMP))
			ipc_PutValue(IPC_MIN_TEMP, temp);

		if (lastSampleCount == 0 || temp > ipc_GetValue(IPC_MAX_TEMP))
			ipc_PutValue(IPC_MAX_TEMP, temp);

		// get ADC min/max
		if (lastSampleCount == 0 || adc < ipc_GetValue(IPC_MIN_ADC))
			ipc_PutValue(IPC_MIN_ADC, adc);

		if (lastSampleCount == 0 || adc > ipc_GetValue(IPC_MAX_ADC))
			ipc_PutValue(IPC_MAX_ADC, adc);

		// get average temperature and ADC value
		n = 0;
		tempTotal = 0;
		adcTotal = 0;
		for (i = 0; i < MAX_SAMPLES; i++)
		{	
			if (sampleData[i].num)
			{
				tempTotal += sampleData[i].temp;
				adcTotal += sampleData[i].adc;
				n++;
			}
		}
		ipc_PutValue(IPC_AVE_TEMP, (tempTotal + n/2) / n);
		ipc_PutValue(IPC_AVE_ADC, (adcTotal + n/2) / n);
		lastSampleCount = sampleCount;
	}
}

//==============================================================================
// This function implements a very simple dynamic HTTP-server.
// It waits until connected, then sends a HTTP-header and the
// HTML-code stored in memory. Before sending, it replaces
// some special strings with dynamic values.
//
// A simple CGI mechanism has been added to provide feedback from the client
//	in this case to set the volume and delay settings
//
//==============================================================================

void HTTPServer(void)
{

	if (SocketStatus & SOCK_CONNECTED)						// check if somebody has connected to our TCP
	{
		if (SocketStatus & SOCK_DATA_AVAILABLE)				// check if remote TCP sent data
		{
			TCPReleaseRxBuffer();							// throw it away
		}

		if (SocketStatus & SOCK_TX_BUF_RELEASED)			// check if buffer is free for TX
		{

			if (!(HTTPStatus & HTTP_SEND_PAGE))				// init byte-counter and pointer to webside
			{												// if called the 1st time
				ReplaceTokens(webSide, webPage);
				PWebSide = (unsigned char *)webSide;
				HTTPBytesToSend = strlen(webSide);
			}

			if (HTTPBytesToSend > MAX_TCP_TX_DATA_SIZE)		// transmit a segment of MAX_SIZE
			{

				if (!(HTTPStatus & HTTP_SEND_PAGE))			// 1st time, include HTTP-header
				{
					memcpy(TCP_TX_BUF, GetResponse, sizeof(GetResponse) - 1);
					memcpy(TCP_TX_BUF + sizeof(GetResponse) - 1, PWebSide, MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1);
					HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
					PWebSide += MAX_TCP_TX_DATA_SIZE - sizeof(GetResponse) + 1;
				}
				else
				{
					memcpy(TCP_TX_BUF, PWebSide, MAX_TCP_TX_DATA_SIZE);
					HTTPBytesToSend -= MAX_TCP_TX_DATA_SIZE;
					PWebSide += MAX_TCP_TX_DATA_SIZE;
				}

				TCPTxDataCount = MAX_TCP_TX_DATA_SIZE;		// bytes to xfer
				TCPTransmitTxBuffer();						// xfer buffer
			}

			else if (HTTPBytesToSend)						// transmit leftover bytes
			{
				memcpy(TCP_TX_BUF, PWebSide, HTTPBytesToSend);
				TCPTxDataCount = HTTPBytesToSend;			// bytes to xfer
				TCPTransmitTxBuffer();						// send last segment
				TCPClose();									// and close connection
				HTTPBytesToSend = 0;						// all data sent
			}
			HTTPStatus |= HTTP_SEND_PAGE;					// ok, 1st loop executed
		}
	}

	else
	{
		HTTPStatus &= ~HTTP_SEND_PAGE;						// reset help-flag if not connected
	}	
}

//==============================================================================
// Replace token values in the web page template with IPC values.
//==============================================================================

void ReplaceTokens(char *strOut, const char *strIn)
{
	char ch, format[10], token[20];
	int i, n;

	while (*strIn)
	{
		// copy string and check for tokens e.g.  %[1] or %[2:%3d] or %[history]
		ch = *strIn++;
		if (ch == '%' && *strIn == '[')
		{
			strIn++;
			ch = *strIn++;

 			if (isdigit(ch))
			{
	 			// get token number
			    n = 0;
				while (isdigit(ch))
				{
					n = (n * 10) + (ch - '0');
					ch = *strIn++;
				}

	 			// check for number format string
				if (ch == ':')
				{
					ch = *strIn++;
	
					i = 0;
					while (ch && (ch != ']') && i < 8)
					{
						format[i++] = ch;
						ch = *strIn++;
					}
					format[i] = '\0';
				}
	
				else
					strcpy(format, "%d");

				// search for end of token
				while (ch && (ch != ']'))
					ch = *strIn++;

				// replace token
				strOut += sprintf(strOut, format, ipc_GetValue(n));
 			}
			else
			{
				// get alphanumeric token
				i = 0;
				while (ch && (ch != ']'))
				{
					if (i < 19) token[i++] = toupper(ch);
					ch = *strIn++;
				}
				token[i] = '\0';

				if (strcmp(token, "HISTORY") == 0)
					strOut = CreateHistory(strOut);

				else if (strcmp(token, "SAMPLE_DATE") == 0)
					strOut = CopyDateTime(strOut, ipc_GetValue(6), ipc_GetValue(7));

				else if (strcmp(token, "ALARM_DATE") == 0)
					strOut = CopyDateTime(strOut, ipc_GetValue(17), ipc_GetValue(18));
			}
		}

		else
			*strOut++ = ch;
	}
	*strOut = '\0';
}

//==============================================================================
// Create sample data history
//==============================================================================

char *CreateHistory(char *sptr)
{
	int i, n, idx;

	// check for no history
	idx = sampleIndex;
	if (sampleData[idx].num == 0)
		return CopyString(sptr, "No History");

	// dispolay the sample history
	for (i = 0; i < MAX_SAMPLES; i++)
	{
	 	idx = (idx + 1) % MAX_SAMPLES;
		// check for valid record
		n = sampleData[idx].num;
		if (n)
		{
			// display data sample history
			sptr = CopyString(sptr, "<table  border=\"1\" cellpadding=\"0\" cellspacing=\"0\"><tr><td width=\"100\">");
			sptr += sprintf(sptr, "%d", n);
			sptr = CopyString(sptr, "</td><td width=\"150\">&nbsp;");
			sptr = CopyDateTime(sptr, sampleData[idx].date, sampleData[idx].time);
			sptr = CopyString(sptr, "</td><td width=\"50\">&nbsp;");
			sptr += sprintf(sptr, "%d", sampleData[idx].temp);
			sptr = CopyString(sptr, "</td><td width=\"50\">&nbsp;");
			n = sampleData[idx].adc;
			sptr += sprintf(sptr, "%d", n);
	
			// display bar chart
			sptr = CopyString(sptr, "</td><td bgcolor=\"#");
	
			// set color of bar chart to green, yellow, red based on ADC value
			if (n < 1024/3)
				sptr = CopyString(sptr, "00ff00");
			else if (n < 1024*2/3)
				sptr = CopyString(sptr, "ffff00");
			else
				sptr = CopyString(sptr, "ff0000");
	
			sptr = CopyString(sptr, "\" width=\"");
			sptr += sprintf(sptr, "%d", n/3);
			sptr = CopyString(sptr, "\">&nbsp</td></tr></table>");
		}
	}
		
	return sptr;	
}

//==============================================================================
// Copy date/time
//==============================================================================

char *CopyDateTime(char *sptr, int date, int time)
{
	if (date && time)
	{
		sptr += sprintf(sptr, "%d-%02d-%02d", (date>>16)&0xFFF, (date>>8)&0xFF, date&0xFF);
		sptr += sprintf(sptr, " %02d:%02d:%02d", (time>>16)&0xFF, (time>>8)&0xFF, time&0xFF);
	}
	return sptr;	
}

//==============================================================================
// Copy string to pointer
//==============================================================================

char *CopyString(char *sptr, char *from)
{
	while (*from)
		*sptr++ = *from++;
	return sptr;	
}


