/**********************************************************************
*
* @file		COG_LCD.c
* @brief	Interface routines for the onboard Chip-on-Glass LCD.
* @version	1.0
* @date		09. January. 2012
* @author	Cam Thompson, Micromega Corporation
*			some routines ported from BW_LCD.c
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include <string.h>
#include "LPC43xx.h"
#include "COG_LCD.h"
#include "COG_LCD_IO.h"
#include "COG_LCD_Font.h"

extern void delay(uint32_t ms);
extern void SPIDisplay_Reset(void);
extern void SPIDisplay_SendCommand(uint8_t data);
extern void SPIDisplay_SendData(uint8_t data);

/* Private Definitions -------------------------------------------------------- */
#define LCD_RAM_OUT_NORMAL	0xA0				// RAM->SEG output = normal
#define LCD_RAM_OUT_REVERSE	0xA1				// RAM->SEG output = reverse
#define LCD_OFF				0xAE				// Display OFF
#define LCD_ON				0xAF				// Display ON
#define LCD_COM_DIR_NORMAL	0xC0				// COM scan direction = normal
#define LCD_COM_DIR_REVERSE	0xC8				// COM scan direction = reverse
#define LCD_BIAS_1_9		0xA2				// 1/9 bias
#define LCD_BIAS_1_7		0xA3				// 1/7 bias
#define LCD_POWER_MODE		0x2F				// power control set
#define LCD_RESISTOR_RATIO	0x20				// resistor ratio set (3 bit)
#define LCD_VOLUME_SET		0x81				// Electronic volume command (set contrast)

#define LCD_SET_START_ADDR	0x40				// Display start address + 6bit
#define LCD_PAGE_BASE_ADDR	0xB0				// Page address + 2bit

#define LCD_NORMAL			0xA6
#define LCD_REVERSE			0xA7

#define XSIZE				128
#define YSIZE				32
#define YPAGES				(YSIZE/8)

#define LCD_RAM_SIZE		(XSIZE*YPAGES)
#define NROWS				4
#define NCOLS				21

#define LCD_RAM_LOCATION	(volatile uint32_t*)(0x18001A08)
#define LCD_RAM_IMAGE		((volatile uint32_t*)0x18001A50)


/* Private Variables ---------------------------------------------------------- */
static uint8_t row;
static uint8_t col;
static uint8_t fontSize = 1;
static uint8_t penType = 1;

/* Private Functions ---------------------------------------------------------- */
static void COG_LCD_WritePutc(char ch);
static void COG_LCD_WriteColumn(uint8_t x, uint32_t value, uint8_t repeat);
static void COG_LCD_WriteCharColumn(uint8_t x, uint8_t y, uint8_t bval);
static void COG_LCD_WriteChar(uint8_t x, uint8_t y, uint8_t ch);

/**********************************************************************
 * @brief		Initialize SPI interface and COG LCD
 * @param[in]	None
 * @return 		None
 *********************************************************************/

void COG_LCD_Init(void)
{
	SPIDisplay_Init();
	SPIDisplay_Reset();
	delay(100);										// wait for hardware reset, at least 2ms
	SPIDisplay_SendCommand(LCD_RAM_OUT_NORMAL);		// RAM->SEG output = normal
	SPIDisplay_SendCommand(LCD_OFF);				// Display OFF
	SPIDisplay_SendCommand(LCD_COM_DIR_REVERSE);	// COM scan direction = reverse
	SPIDisplay_SendCommand(LCD_BIAS_1_9);			// 1/9 bias
	SPIDisplay_SendCommand(LCD_POWER_MODE);			// power control set
	SPIDisplay_SendCommand(LCD_RESISTOR_RATIO + 1);	// resistor ratio set
	SPIDisplay_SendCommand(LCD_VOLUME_SET);			// Electronic volume command (set contrast)
	SPIDisplay_SendCommand(0x2F);					// Electronic volume value (contrast value)
	SPIDisplay_SendCommand(LCD_ON);

	COG_LCD_Clear();
}

/**********************************************************************
 * @brief		Clear COG LCD display buffer and display
 * @param[in]	None
 * @return 		None
 *********************************************************************/

void COG_LCD_Clear(void)
{
	COG_LCD_DrawClear();
	COG_LCD_Update();
}

/**********************************************************************
 * @brief		Clear COG LCD display buffer
 * @param[in]	None
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawClear(void)
{
	memset((void *)LCD_RAM_IMAGE, 0, (size_t) LCD_RAM_SIZE);
	row = 0;
	col = 0;
}

/**********************************************************************
 * @brief		Set pen type 
 * @param[in]	0-Set, 1-OR, 2-AND, 3-XOR
 * @return 		None
 *********************************************************************/

void COG_LCD_PenType(uint8_t n)
{
	penType = n;
}

/**********************************************************************
 * @brief		Set font size
 * @param[in]	1 to 4
 * @return 		None
 *********************************************************************/

void COG_LCD_FontSize(uint8_t n)
{
	if (n < 1)
		n = 1;
	else if (n > 4)
		n = 4;
	fontSize = n;
}

/**********************************************************************
 * @brief		Set COG LCD display to black on white (default)
 * @param[in]	None
 * @return 		None
 *********************************************************************/

void COG_LCD_BlackOnWhite(void)
{
	SPIDisplay_SendCommand(LCD_NORMAL);
}

/**********************************************************************
 * @brief		Set COG LCD display to white on black (reverse)
 * @param[in]	None
 * @return 		None
 *********************************************************************/

void COG_LCD_WhiteOnBlack(void)
{
	SPIDisplay_SendCommand(LCD_REVERSE);
}

/**********************************************************************
 * @brief		Set row, column 
 * @param[in]	row
 * @param[in]	column
 * @return 		None
 *********************************************************************/

void COG_LCD_Move(uint8_t rowVal, uint8_t colVal)
{
	row = rowVal;
	if (row > NROWS) row = NROWS-1;
	col = colVal;
	if (col > NCOLS) col = NCOLS-1;
}

/**********************************************************************
 * @brief		Print string starting at current row, column 
 * @param[in]	string
 * @return 		None
 *********************************************************************/

void COG_LCD_Print(char *s)
{
	while (*s) COG_LCD_WritePutc(*s++);
   	COG_LCD_Update();
}

/*********************************************************************
 * @brief		Print character at current row, column 
 * @param[in]	string
 * @return 		None
 *********************************************************************/

void COG_LCD_Putc(char ch)
{
	COG_LCD_WritePutc(ch);
	COG_LCD_Update();
}

static void COG_LCD_WritePutc(char ch)
{
	fontSize = 1;

	if (ch == '\f')
	{
		COG_LCD_DrawClear();
	}
	else if (ch == '\r')
	{
		row++;
		col = 0;
		if (row >= NROWS) row = 0;
	}
	else
	{
		penType = PEN_CLEAR;
		COG_LCD_WriteColumn(col*6, 0xFF << (row*8), 6);
		penType = PEN_SET;
		COG_LCD_WriteChar(col*6, row*8, ch);

		col++;
		if (col >= NCOLS)
		{
			col = 0;
			row++;
			if (row >= NROWS) row = 0;
		}
	}

	penType = PEN_SET;
}

/*********************************************************************
 * @brief		Draw point at x, y 
 * @param[in]	x
 * @param[in]	y
 * @return 		None
 *********************************************************************/
  
void COG_LCD_DrawPoint(uint8_t x, uint8_t y)
{
    COG_LCD_WriteColumn(x, 1 << y, 1);
}

/*********************************************************************
 * @brief		Draw line from x1,y1 to x2, y2
 * @param[in]	x1
 * @param[in]	y1
 * @param[in]	x2
 * @param[in]	y2
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawLine(uint8_t x1, uint8_t y1,uint8_t x2, uint8_t y2)
{
	uint8_t xdiff, ydiff, n, frac, len, nbits;

	if (x1 > x2)
		xdiff = (x1 - x2) + 1;
	else
		xdiff = (x2 - x1) + 1;

	if (y1 > y2)
		ydiff = (y1 - y2) + 1;
	else
		ydiff = (y2 - y1) + 1;

	if (xdiff == 1)
	{
		if (y1 > y2) y1 = y2;
		COG_LCD_WriteColumn(x1, (~(0xFFFFFFFF << ydiff)) << y1, 1);
	}
	else if (ydiff == 1)
	{
		if (x1 > x2) x1 = x2;
    	COG_LCD_WriteColumn(x1, 1 << y1, xdiff);
	}
	else if (xdiff > ydiff)
	{
		if (x1 > x2)
		{
			x1 = x2;
			n = y1;
			y1 = y2;
			y2 = n;
		}
		nbits = xdiff / ydiff;
		frac = (xdiff % ydiff) * 128 / ydiff;
		n = frac;
		while (xdiff > 0)
		{
			len = nbits;
			n += frac;
			if (n >= 128)
			{
				len++;
				n -= 128;
			}
			if (len > xdiff) len = xdiff;
			COG_LCD_WriteColumn(x1, 1 << y1, len);
			x1 += len;
			xdiff -= len;
			if (y1 > y2)
				y1--;
			else if (y1 < y2)
				y1++;
		}
	}
	else
	{
		if (y1 > y2)
		{
			y1 = y2;
			n = x1;
			x1 = x2;
			x2 = n;
		}
		nbits = ydiff / xdiff;
		frac = (ydiff % xdiff) * 128 / xdiff;
		n = frac;
		while (ydiff > 0)
		{
			len = nbits;
			n += frac;
			if (n >= 128)
			{
				len++;
				n -= 128;
			}
			if (len > ydiff) len = ydiff;
			COG_LCD_WriteColumn(x1, (~(0xFFFFFFFF << len)) << y1, 1);
			y1 += len;
			ydiff -= len;
			if (x1 > x2)
				x1--;
			else if (x1 < x2)
				x1++;
		}
	}
}

/*********************************************************************
 * @brief		Draw rectangle at x, y with specified width, height
 * @param[in]	x		top left corner of rectangle
 * @param[in]	y
 * @param[in]	width
 * @param[in]	height
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawRect(uint8_t x, uint8_t y, uint8_t width, uint8_t height)
{
	uint32_t value;

	value = (~(0xFFFFFFFF << height)) << y;
	COG_LCD_WriteColumn(x, value, 1);
	if (width > 2) COG_LCD_WriteColumn(x+1, (1 << y) | (1 << (y+height-1)), width-2);
	if (width > 1) COG_LCD_WriteColumn(x+width-1, value, 1);
}

/*********************************************************************
 * @brief		Draw filled rectangle at x, y with specified width, height
 * @param[in]	x		top left corner of rectangle
 * @param[in]	y
 * @param[in]	width
 * @param[in]	height
 * @return 		None
 *********************************************************************/

void COG_LCD_FillRect(uint8_t x, uint8_t y, uint8_t width, uint8_t height)
{
	COG_LCD_WriteColumn(x, (~(0xFFFFFFFF << height)) << y, width);
}

/*********************************************************************
 * @brief		Draw filled rectangle at x, y with specified width, height
 * @param[in]	x		top left corner of rectangle
 * @param[in]	y
 * @param[in]	width
 * @param[in]	height
 * @param[in]	byte array
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawImage(uint8_t x, uint8_t y, uint8_t width, uint8_t height, uint8_t *b)
{
	uint32_t value;
	uint8_t i, j;

	height /= 8;
	for (i = 0; i < width; i++)
	{
		value = 0;
		for (j = 0; j < height; j++)
			value |= (b[i+j*width] << (y+j*8));
		COG_LCD_WriteColumn(x+i, value, 1);
	}
}

/*********************************************************************
 * @brief		Draw string at x, y
 * @param[in]	x		top left corner of string
 * @param[in]	y
 * @param[in]	string
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawString(uint8_t x, uint8_t y, char *s)
{
	while (*s)
	{
		COG_LCD_WriteChar(x, y, *s++);
		x += fontSize * 6;
	}
}

/*********************************************************************
 * @brief		Draw byte string at x, y
 * @param[in]	x		top left corner of byte string
 * @param[in]	y
 * @param[in]	byte array length
 * @param[in]	byte array
 * @return 		None
 *********************************************************************/

void COG_LCD_DrawByteString(uint8_t x, uint8_t y, uint8_t len, uint8_t *b)
{
	while (len--)
	{
		COG_LCD_WriteCharColumn(x, y, *b++);
		x += fontSize;
	}
}

/*********************************************************************
 * @brief		Update the COG LCD from display memory
 * @param[in]	x		top left corner of string
 * @param[in]	y
 * @param[in]	string
 * @return 		None
 *********************************************************************/

void COG_LCD_Update(void)
{
	uint8_t i;
	uint8_t	j;
	uint8_t *lcd_pix 	= (uint8_t *)LCD_RAM_IMAGE;
	uint8_t page 		= LCD_PAGE_BASE_ADDR;	//Page Address + 0xB0

	delay(100);
//	SPIDisplay_SendCommand(LCD_OFF);				//Display OFF
	SPIDisplay_SendCommand(LCD_SET_START_ADDR);		//Display start address + 0x40

	for(i = 0; i < YPAGES; i++ )
	{ /* 32 pixel display / 8 pixels per page = 4 pages */
		SPIDisplay_SendCommand(page);				//send page address
		SPIDisplay_SendCommand(0x10);				//column address upper 4 bits + 0x10
		SPIDisplay_SendCommand(0x00);				//column address lower 4 bits + 0x00

		for(j = 0; j < XSIZE; j++)
		{				//128 columns wide
			SPIDisplay_SendData(*lcd_pix);			//send picture data
			lcd_pix++;								//point to next picture data
		}
		page++;										//after 128 columns, go to next page
	}

//	SPIDisplay_SendCommand(LCD_ON);					//Display ON
}

/*********************************************************************
 * @brief		Load character from font and store to display memory
 * @param[in]	x		top left corner of string
 * @param[in]	y
 * @param[in]	string
 * @return 		None
 *********************************************************************/

static void COG_LCD_WriteChar(uint8_t x, uint8_t y, uint8_t ch)
{
	uint8_t i;

	if (ch < 0x20 || ch > 0x7F) return;

	for (i = 0; i < 5; i++)
	{
		COG_LCD_WriteCharColumn(x, y, COG_LCD_FONT[(ch - 0x20)* 5 + i]);
		x += fontSize;
	}
}

/*********************************************************************
 * @brief		Load character column from font and store in display memory
 * @param[in]	x		top left corner of string
 * @param[in]	y
 * @param[in]	string
 * @return 		None
 *********************************************************************/

static void COG_LCD_WriteCharColumn(uint8_t x, uint8_t y, uint8_t bval)
{
	uint32_t value;
	uint8_t y2;

	if (fontSize == 1)
	{
		value = bval;
	}
	else
	{
		value = 0;
		y2 = 0;
		while (bval)
		{
			if (bval & 0x01) value |= (~(0xFFFFFFFF << fontSize)) << y2;
			y2 += fontSize;
			bval >>= 1;
		}
	}
	COG_LCD_WriteColumn(x, value<<y, fontSize);
}

/*********************************************************************
 * @brief		Write column of data to display memory
 * @param[in]	x		top left corner of string
 * @param[in]	value
 * @param[in]	repeat
 * @return 		None
 *********************************************************************/

static void COG_LCD_WriteColumn(uint8_t x, uint32_t value, uint8_t repeat)
{
	uint8_t yPage, bval, *dataPtr, i;

	if (x >= XSIZE) return;
	yPage = 0;

	while (value)
	{
		bval = value & 0xFF;
		if (bval)
		{
			dataPtr = (uint8_t *)LCD_RAM_IMAGE + (yPage * XSIZE) + x;
			for (i = 0; i < repeat; i++)
			{
				if (x+i >= XSIZE) break;

				if (penType == PEN_CLEAR)
					*dataPtr &= ~bval;

				else if (penType == PEN_TOGGLE)
					*dataPtr ^= bval;

				else
					*dataPtr |= bval;
				dataPtr++;
			}
		}
		yPage++;
		value >>= 8;
	}
}

