/**********************************************************************
*
* @file		COG_LCD_IO.h
* @brief	COG LCD I/O routines.
* @version	1.0
* @date		09. January. 2012
* @author	Cam Thompson, Micromega Corporation
*			ported from BW_LCDConfig.c
*
**********************************************************************/

#include "LPC43xx.h"
#include "LPC43xx_scu.h"
#include "LPC43xx_ssp.h"
#include "LPC43xx_gpio.h"
#include "COG_LCD_IO.h"

#define inline __inline

static inline void SPIDisplay_SetA0(void)
{
	GPIO_SetValue(4, 1<<10);
}

static inline void SPIDisplay_ClrA0(void)
{
	GPIO_ClearValue(4, 1<<10);
}

static inline void SPIDisplay_SetCS(void)
{
	GPIO_SetValue(3, 1<<12);
}

static inline void SPIDisplay_ClrCS(void)
{
	GPIO_ClearValue(3, 1<<12);
}

static inline void SPIDisplay_SetRST(void)
{
	GPIO_SetValue(4, 1<<9);
}

static inline void SPIDisplay_ClrRST(void)
{
	GPIO_ClearValue(4, 1<<9);
}

/* used register bits*/
#define FIFOSIZE		8
#define SSPCR1_SSE		(1 << 1)

/* status register bits*/
#define SSPSR_TFE		(1 << 0)
#define SSPSR_TNF		(1 << 1) 
#define SSPSR_RNE		(1 << 2)
#define SSPSR_RFF		(1 << 3) 
#define SSPSR_BSY		(1 << 4)

/* Private Functions ---------------------------------------------------------- */
static inline void SSPInit(void);

void SPIDisplay_Init(void)		 		
{
	scu_pinmux(0xA ,2 , MD_PDN, FUNC0); // PA.2: LCD_RST
	scu_pinmux(0x7 ,4 , MD_PDN, FUNC0); // P7.4: LCD_CS
	scu_pinmux(0xA ,3 , MD_PDN, FUNC0); // PA.3: LCD_A0

	GPIO_SetDir(4, 1<<9, 1);
	GPIO_SetDir(3, 1<<12, 1);
	GPIO_SetDir(4, 1<<10, 1);

	scu_pinmux(0xF, 0, MD_PLN,FUNC0);	// PF.0 connected to nSI		func0=SSP0 SCK0	
	scu_pinmux(0xF, 2, MD_PLN,FUNC0);	// PF.2 connected to SCL/SCLK	func0=SSP0 MISO0
	scu_pinmux(0xF, 3, MD_PLN,FUNC2);	// PF.3 connected to SO			func2=SSP0 MOSI0

	SSPInit();
}


static inline void SSPInit(void)
{
	uint8_t i, Dummy=Dummy;

	/* 	Set DSS data to 8-bit, Frame format SPI, 
		CPOL = 0, CPHA = 0, and SCR is 3; divide by 4 to get 100kHz. */
	LPC_SSP0->CR0 = (0x0307 | (1 << 6) | (1 << 7));	

	/* setup Clock*/
	LPC_SSP0->CPSR = 30;

	/* clear the RxFIFO */
	for ( i = 0; i < FIFOSIZE; i++ )
	{
		Dummy = LPC_SSP0->DR;		
	}
	
	/* Enable SSP peripheral */
	LPC_SSP0->CR1 |= SSPCR1_SSE;
}


static void SSPSendReceive(uint8_t* buffer, uint32_t count)
{
	uint32_t i;
	uint8_t Dummy = Dummy;

	for ( i = 0; i < count; i++ )
	{
		/* Move on only if NOT busy and TX FIFO not full. */
		while ( (LPC_SSP0->SR & (SSPSR_TNF|SSPSR_BSY)) != SSPSR_TNF );
		/* Write data byte */
		LPC_SSP0->DR = *buffer;
		/* Wait for write to finish */
		while(LPC_SSP0->SR & SSPSR_BSY);

		/* Read byte from buffer and increment buffer pointer */
		*buffer++ = LPC_SSP0->DR;
	}
}


void SPIDisplay_SendData(uint8_t data) 			
{
 	SPIDisplay_ClrCS();		  		// Chip Select = Active
	SPIDisplay_SetA0();				// A0 = Data
	SSPSendReceive(&data,1);
	SPIDisplay_SetCS();				// after 1 byte, Chip Select = inactive
}


void SPIDisplay_SendCommand(uint8_t data) 	
{
	SPIDisplay_ClrCS();		  		// Chip Select = Active
	SPIDisplay_ClrA0();				// A0 = Command
	SSPSendReceive(&data,1);
	SPIDisplay_SetCS();				// after 1 byte, Chip Select = inactive
}


void SPIDisplay_Reset(void)
{
	SPIDisplay_ClrRST();			// reset signal
	SPIDisplay_SetRST();
}


void delay(uint32_t count) 
{
	volatile uint32_t i;
	for(i = 0; i < 10000*count; i++);
}
