/**********************************************************************
*
* @file		main.c
* @brief	Main routine for Web Data Logging demo.
* @version	1.0
* @date		28. January. 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

#ifndef CORE_M4
	  #error "Build Error: please define CORE_M4 in the project settings" 
#endif

#include "LPC43xx.h"
#include "lpc43xx_rtc.h"
#include "lpc43xx_evrt.h"
#include "lpc43xx_adc.h"

#include "platform_config.h"
#include "ipc_mbx.h"
#include "ipc_memory.h"
#include "platform_check.h"
#include "pca9502.h"
#include "COG_LCD.h"
#include "se97.h"
#include "demo_utilities.h"

#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "stdio.h"
#include "ctype.h"

//-------------------------- Private Definitions -------------------------------

#define _ADC_INT			ADC_ADINTEN0
#define _ADC_CHANNEL		ADC_CHANNEL_0

#define CLOCK_SPEED (120000000UL)

#define ALARM_SET	10
#define ALARM_CLEAR	1

//-------------------------- Private Variables ---------------------------------


static volatile uint32_t u32Milliseconds = 1500;

Bool	tick, alarm;			// RTC flags
uint32_t msec; 					// delay counter

//-------------------------- Local Prototypes ----------------------------------

void Delay_msec(uint32_t delay);

//-------------------------- M0 Image File -------------------------------------
/* fromelf.exe always generates an unsigned char LR0[] type of array */
const	/* DO NOT REMOVE THIS CONST QUALIFIER, IS USED TO PLACE THE IMAGE IN M4 ROM */
#include SLAVE_IMAGE_FILE

//==============================================================================
//   Main Function (M4)
//==============================================================================

int main (void) 
{
	int n;
	char sbuf[20];
	uint32_t date, time, temp, adcValue;

	// initialize platform dependent things - I/O, clock, fpu etc
	platformInit();

	// initialize COG, Touch Keys, LEDs, Temperature sensor, RTC
	COG_LCD_Init();
	COG_LCD_Print("Starting up M4/M0...");
	pca9502_Init();
	se97_Init();
	RTC_Init(LPC_RTC);

	// initialize ADC
	ADC_Init(LPC_ADC0, 200000, 10);
	ADC_IntConfig(LPC_ADC0,_ADC_INT,DISABLE);
	ADC_ChannelCmd(LPC_ADC0,_ADC_CHANNEL,ENABLE);

 	/* Set ALARM at :15 second mark of every minute */
	RTC_SetAlarmTime (LPC_RTC, RTC_TIMETYPE_SECOND, 15);
	RTC_AlarmIntConfig (LPC_RTC, RTC_TIMETYPE_SECOND, ENABLE);

	/* Set the counter increment interrupt for 1 second interrupts */
	RTC_CntIncrIntConfig (LPC_RTC, RTC_TIMETYPE_SECOND, ENABLE);

 	// set event routing for RTC wakeup event
	EVRT_Init(LPC_EVENTROUTER);
	EVRT_ConfigIntSrcActiveType(LPC_EVENTROUTER,EVRT_SRC_RTC,EVRT_SRC_ACTIVE_RISING_EDGE);
	EVRT_SetUpIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC,ENABLE);
	EVRT_ClrPendIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC);

    // Enable EVRT interrupt
    NVIC_DisableIRQ(EVENTROUTER_IRQn);
    NVIC_SetPriority(EVENTROUTER_IRQn, ((0x01<<3)|0x01));	// preemption = 1, sub-priority
    NVIC_EnableIRQ(EVENTROUTER_IRQn);

	// set system ticks at 1 kHz, period = 1 msec
	SysTick_Config(CLOCK_SPEED/1000);				

	// stop the M0 processor
	IPC_haltSlave();

	// download the M0 cpu image
	IPC_downloadSlaveImage(SLAVE_ROM_START, &LR0[0], sizeof(LR0));

	// initialize the ipc area
	ipc_Init();

	// start the M0 processor
	IPC_startSlave();

 	// wait for M0 to start
	COG_LCD_Clear();
	COG_LCD_Print("Waiting for M0...");
	while (ipc_GetValue(IPC_M0_COUNTER) == 0) ;
		 
	// set RTC date/time
	SetDateTime();
	
 	// display title and IP address
	COG_LCD_Clear();
	COG_LCD_PenType(PEN_SET);
	COG_LCD_FillRect(0, 0, 128, 8);
	COG_LCD_PenType(PEN_CLEAR);
	COG_LCD_DrawString(17, 0, "Web Data Logging");
	COG_LCD_PenType(PEN_SET);
	COG_LCD_DrawString(0, 8, "IP:");
	n = ipc_GetValue(IPC_IP_ADDRESS);
	sprintf(sbuf, "%d.%d.%d.%d", (n>>24)&0xFF, (n>>16)&0xFF, (n>>8)&0xFF, n&0xFF);
	COG_LCD_DrawString(20, 8, sbuf);

	// data sampling loop
	while(1)
	{
		// log data sample
		if (tick)
		{
			// get date/time
			date = DateValue();
			time = TimeValue();

			// get temperature
			temp = se97_Temperature();

			// get ADC0 value
			ADC_StartCmd(LPC_ADC0, ADC_START_NOW);
			while (!(ADC_ChannelGetStatus(LPC_ADC0, _ADC_CHANNEL, ADC_DATA_DONE)));
			adcValue = ADC_ChannelGetData(LPC_ADC0, _ADC_CHANNEL);

			// store data sample to ipc memory			 	
			ipc_StartWrite();
			ipc_IncrementValue(IPC_SAMPLE_COUNT);
			ipc_PutValue(IPC_SAMPLE_DATE, date);
			ipc_PutValue(IPC_SAMPLE_TIME, time);
			ipc_PutValue(IPC_TEMP, temp);
			ipc_PutValue(IPC_ADC, adcValue);
			ipc_EndWrite();

  			// display the sample values on COG LCD
			COG_LCD_PenType(PEN_CLEAR);
			COG_LCD_FillRect(0, 15, 127, 16);

			COG_LCD_PenType(PEN_SET);
			COG_LCD_DrawString(0, 16, DateTimeString(date, time));
			COG_LCD_DrawString(0, 24, "Temp:");
			COG_LCD_DrawString(32, 24, itoa(temp, sbuf, 3));
			COG_LCD_DrawString(66, 24, "ADC:");
			COG_LCD_DrawString(92, 24, itoa(adcValue, sbuf, 4));
			COG_LCD_Update();
			tick = FALSE;
		}

		// log alarm
		if (alarm)
		{
			// get date/time
			date = DateValue();
			time = TimeValue();

			// store alarm to ipc memory			 	
			ipc_StartWrite();
			ipc_IncrementValue(IPC_ALARM_COUNT);
			ipc_PutValue(IPC_ALARM_DATE, date);
			ipc_PutValue(IPC_ALARM_TIME, time);
			ipc_EndWrite();
			alarm = FALSE;
		}
	}
}

//==============================================================================
// System tick interrupt handler
//==============================================================================

void SysTick_Handler (void) 					
{           
	if(u32Milliseconds > 0)
	{
		u32Milliseconds--; 
	}
	else
	{
		M4_COUNTER = M4_COUNTER + 1;			// increment M4 interrupt counter

		#if (PLATFORM == NXP_VALIDATION_BOARD)	// toggle M4 LED
			LPC_GPIO_PORT->NOT[4] |= (1UL << 14);
		#endif
		#if (PLATFORM == HITEX_A2_BOARD)					
			LPC_GPIO_PORT->NOT[4] |= (1UL << 15);    
		#endif
		u32Milliseconds = 1000;	
	}

	if (msec) msec--;
}

//==============================================================================
// EVRT interrupt handler sub-routine
//==============================================================================

void EVRT_IRQHandler(void)
{
	if(EVRT_IsSourceInterrupting(LPC_EVENTROUTER,EVRT_SRC_RTC))
	{
		if (RTC_GetIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE))
		{
			RTC_ClearIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE);
			tick = TRUE;
		}

		if (RTC_GetIntPending(LPC_RTC, RTC_INT_ALARM))
		{
			RTC_ClearIntPending(LPC_RTC, RTC_INT_ALARM);
			alarm = TRUE;
		}

		EVRT_ClrPendIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC);
	}
}

//==============================================================================
// Delay (in msec)
//==============================================================================

void Delay_msec(uint32_t delay)
{
	msec = delay;
	while(msec) ;
}
