/**********************************************************************
*
* @file		pca9502.c
* @brief	This provides interface routines for the PCA9502,
*			controlling the four touch keys and LEDs
* @version	1.1
* @date		10. January. 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "pca9502.h"
#include "lpc43xx_i2c.h"

/* Private Variables ---------------------------------------------------------- */

static I2C_M_SETUP_Type i2cMaster;
static uint8_t	i2cWriteBuffer[2];
static uint8_t	i2cReadBuffer[2];

/* Private Functions ---------------------------------------------------------- */

static void pca9502_Write(uint8_t reg, uint8_t val);
static uint8_t pca9502_Read(uint8_t reg);

/**********************************************************************
 * @brief		Write byte to PCA9502 register
 * @param[in]	PCA9502 register number
 * @param[in]	value to write
 * @return 		None
 *********************************************************************/
static void pca9502_Write(uint8_t reg, uint8_t val)
{
	// store values in write buffer
	i2cWriteBuffer[0] = reg;
	i2cWriteBuffer[1] = val;

	// transfer the data
	i2cMaster.sl_addr7bit = PCA9502_ADDR;
	i2cMaster.tx_data = i2cWriteBuffer;
	i2cMaster.tx_length = 2;
	i2cMaster.rx_data = NULL;
	i2cMaster.rx_length = 0;
	i2cMaster.retransmissions_max = 1;
	I2C_MasterTransferData(LPC_I2C0, &i2cMaster, I2C_TRANSFER_POLLING);
}

/**********************************************************************
 * @brief		Read byte from PCA9502 register
 * @param[in]	PCA9502 register number
 * @return 		value read from register
 *********************************************************************/
static uint8_t pca9502_Read(uint8_t reg)
{
	// store value in write buffer
	i2cWriteBuffer[0] = reg;

	// transfer the data
	i2cMaster.sl_addr7bit = PCA9502_ADDR;
	i2cMaster.tx_data = i2cWriteBuffer;
	i2cMaster.tx_length = 1;
	i2cMaster.rx_data = i2cReadBuffer;
	i2cMaster.rx_length = 1;
	i2cMaster.retransmissions_max = 1;
	I2C_MasterTransferData(LPC_I2C0, &i2cMaster, I2C_TRANSFER_POLLING);

	return i2cReadBuffer[0];
}

/*--------------------------------------------------------------------------------*/

/**********************************************************************
 * @brief		Initialize PCA9502
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void pca9502_Init(void)
{
	// initialize I2C0
	I2C_Init(LPC_I2C0, 100000);
	I2C_Cmd(LPC_I2C0, ENABLE);

	// send reset to PCA9502
	pca9502_Write(PCA9502_IOCTRL, PCA9502_RESET);
	
  	// set pin direction for PCA9502
	pca9502_Write(PCA9502_IODIR, 0x0F);
}

/**********************************************************************
 * @brief		Write value to PCA9502 LEDs
 * @param[in]	bit3-LED0, bit2-LED1, bit1-LED2, bit0-LED3
  				if bit=0, LED On, if bit=1, LED Off
 * @return 		None
 *********************************************************************/
void pca9502_WriteLeds(uint8_t led)
{
	pca9502_Write(PCA9502_IOSTATE, led & 0x0F);
}

/**********************************************************************
 * @brief		Turn on PCA9502 LEDs
 * @param[in]	bit3-LED0, bit2-LED1, bit1-LED2, bit0-LED3, if bitn=1, turn on LED
  				use symbols: PCA9502_LED0, PCA9502_LED1, PCA9502_LED2, PCA9502_LED3
 * @return 		None
 *********************************************************************/
void pca9502_SetLed(uint8_t led)
{
	uint8_t current;

	current = pca9502_Read(PCA9502_IOSTATE);
	pca9502_Write(PCA9502_IOSTATE, (current & ~led) & 0x0F);
}

/**********************************************************************
 * @brief		Turn off PCA9502 LEDs
 * @param[in]	bit3-LED0, bit2-LED1, bit1-LED2, bit0-LED3, if bitn=1, turn off LED
  				use symbols: PCA9502_LED0, PCA9502_LED1, PCA9502_LED2, PCA9502_LED3
 * @return 		None
 *********************************************************************/
void pca9502_ClearLed(uint8_t led)
{
	uint8_t current;

	current = pca9502_Read(PCA9502_IOSTATE);
	pca9502_Write(PCA9502_IOSTATE, (current | led) & 0x0F);
}

/**********************************************************************
 * @brief		Toggle PCA9502 LEDs
 * @param[in]	bit3-LED0, bit2-LED1, bit1-LED2, bit0-LED3, if bitn=1, toggle LED
  				use symbols: PCA9502_LED0, PCA9502_LED1, PCA9502_LED2, PCA9502_LED3
 * @return 		None
 *********************************************************************/
void pca9502_ToggleLed(uint8_t led)
{
	uint8_t current;

	current = pca9502_Read(PCA9502_IOSTATE);
	pca9502_Write(PCA9502_IOSTATE, (current ^ led) & 0x0F);
}

/**********************************************************************
 * @brief		Read the PCA9502 touch keys
 * @param[in]	bit3-T1, bit2-T2, bit1-T3, bit0-T4, if bitn=1, touch active
  				use symbols: PCA9502_T1, PCA9502_T2, PCA9502_T3, PCA9502_T4
 * @return 		None
 *********************************************************************/
uint8_t ReadTouch(void)
{
	return pca9502_Read(PCA9502_IOSTATE) >> 4;
}

/**********************************************************************
 * @brief		Wait for the specified PCA9502 touch key to be pressed
 * @param[in]	bit3-T1, bit2-T2, bit1-T3, bit0-T4, if bitn=1, touch active
  				use symbols: PCA9502_T1, PCA9502_T2, PCA9502_T3, PCA9502_T4
								PCA9502_TANY
 * @return 		None
 *********************************************************************/
uint8_t WaitTouch(uint8_t touch)
{
	uint8_t touchKeys;

	// wait until no keys pressed
	while (pca9502_Read(PCA9502_IOSTATE) >> 4);

	// wait until one of selected key is pressed
	while(1)
	{
		touchKeys = pca9502_Read(PCA9502_IOSTATE) >> 4;
		pca9502_WriteLeds(~touchKeys);
		if (touchKeys & touch) return touchKeys;
	}
}
