/**********************************************************************
*
* @file		SE97.c
* @brief	This provides interface routines for the SE97
*		temperature sensor and 256 byte EEPROM.
* @version	1.0
* @date		13. January. 2012
* @author	Cam Thompson, Micromega Corporation
*
**********************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "se97.h"
#include "lpc43xx_i2c.h"

/* Private Variables ---------------------------------------------------------- */

static I2C_M_SETUP_Type i2cMaster;
static uint8_t	i2cWriteBuffer[3];
static uint8_t	i2cReadBuffer[3];


/**********************************************************************
 * @brief		Initialize SE97
 * @param[in]	None
 * @return 		None
 *********************************************************************/
void se97_Init(void)
{
	// initialize I2C0
	I2C_Init(LPC_I2C0, 100000);
	I2C_Cmd(LPC_I2C0, ENABLE);
}

/**********************************************************************
 * @brief		Read temperature from SE97 and return integer value
 * @param[in]	SE97 register number
 * @return 		value read from register
 *********************************************************************/
uint16_t se97_Temperature(void)
{
	uint16_t n;

	n = se97_Read(SE97_TEMP_REG) & 0x3FFF;
	if (n & 0x0008) n += 0x0010;
	return n>>4;
}

/**********************************************************************
 * @brief		Write byte to SE97 temperature register
 * @param[in]	SE97 register number
 * @param[in]	value to write
 * @return 		None
 *********************************************************************/
void se97_Write(uint8_t reg, uint16_t val)
{
	// store values in write buffer
	i2cWriteBuffer[0] = reg;
	i2cWriteBuffer[1] = val>>8;
	i2cWriteBuffer[2] = val & 0xFF;

	// transfer the data
	i2cMaster.sl_addr7bit = SE97_TEMP_ADDR;
	i2cMaster.tx_data = i2cWriteBuffer;
	i2cMaster.tx_length = 3;
	i2cMaster.rx_data = NULL;
	i2cMaster.rx_length = 0;
	i2cMaster.retransmissions_max = 1;
	I2C_MasterTransferData(LPC_I2C0, &i2cMaster, I2C_TRANSFER_POLLING);
}

/**********************************************************************
 * @brief		Read byte from SE97 register
 * @param[in]	SE97 register number
 * @return 		value read from register
 *********************************************************************/
uint16_t se97_Read(uint8_t reg)
{
	// store value in write buffer
	i2cWriteBuffer[0] = reg;

	// transfer the data
	i2cMaster.sl_addr7bit = SE97_TEMP_ADDR;
	i2cMaster.tx_data = i2cWriteBuffer;
	i2cMaster.tx_length = 1;
	i2cMaster.rx_data = i2cReadBuffer;
	i2cMaster.rx_length = 2;
	i2cMaster.retransmissions_max = 1;
	I2C_MasterTransferData(LPC_I2C0, &i2cMaster, I2C_TRANSFER_POLLING);

	return (i2cReadBuffer[0]<<8) + i2cReadBuffer[1];
}
